@@ -8,7 +8,7 @@ import lurklite.tempcmds as tempcmds
 static_cmds = None
 
 # The version
-miniirc.version = f'lurklite v0.4.20 (powered by {miniirc.version})'
+miniirc.version = f'lurklite v0.4.22 (powered by {miniirc.version})'
 
 # Throw errors
 class BotError(Exception):
@@ -64,6 +64,11 @@ class Bot:
         p = {}
         self._prefs[irc] = p
 
+        # Add message handlers
+        irc.Handler('PRIVMSG', colon=False)(self.handle_privmsg)
+        if section != 'discord':
+            irc.Handler('INVITE', colon=False)(self._handle_invite)
+
         # Process the ignores list
         if 'ignored' in c:
             p['ignored'] = self.process_ignores(section)
@@ -87,7 +92,7 @@ class Bot:
             return
 
         # Handle PMs correctly
-        if args[0].lower() == irc.nick.lower():
+        if args[0].lower() == irc.current_nick.lower():
             args[0] = hostmask[0]
 
         # [off] handling
@@ -119,7 +124,7 @@ class Bot:
             irc.msg(args[0], reply_prefix + '\u200bYay!')
         elif not self.disable_ouch and msg.startswith('ouch'):
             irc.msg(args[0], reply_prefix + '\u200bOuch.')
-        elif msg.startswith(irc.nick.lower() + '!'):
+        elif msg.startswith(irc.current_nick.lower() + '!'):
             irc.msg(args[0], reply_prefix + hostmask[0] + '!')
         else:
             # "Static" commands
@@ -171,6 +176,12 @@ class Bot:
             irc.debug('Updated Discord status text.')
             self._prefs[irc]['next_update'] = time.time() + 60
 
+    # Accept invites from admins
+    def _handle_invite(self, irc, hostmask, args):
+        admins = self._prefs[irc].get('admins', ())
+        if hostmask[2] in admins:
+            irc.send('JOIN', args[-1])
+
     # The init function
     def __init__(self, config, *, debug=False):
         self.config = config
@@ -256,7 +267,7 @@ class Bot:
 
             if getattr(miniirc_discord, 'ver', ()) < (0,5,18):
                 print('Support for this version of miniirc_discord will be '
-                    'removed in the future.')
+                      'removed in the future.')
                 kw = {}
             else:
                 kw = {'stateless_mode': True}
@@ -274,17 +285,34 @@ class Bot:
             self._add_extras('discord', c, irc)
             self._prefs[irc]['next_update'] = 0
 
+        if 'matrix' in config:
+            try:
+                import miniirc_matrix
+            except ImportError:
+                err('miniirc_matrix is not installed, and a Matrix account'
+                    ' has been specified in the config file!')
+
+            self._conf_assert('matrix', 'homeserver', 'token')
+            c = config['matrix']
+            irc = miniirc_matrix.Matrix(c['homeserver'], auto_connect=False,
+                                        debug=debug, token=c['token'])
+            _servers['Matrix'] = irc
+            self._add_extras('matrix', c, irc)
+
         # Mass connect
-        for name in _servers:
-            irc = _servers[name]
+        for name, irc in _servers.items():
             irc.debug('Connecting to ' + repr(name) + '...')
-            irc.Handler('PRIVMSG', colon=False)(self.handle_privmsg)
             try:
                 irc.connect()
             except Exception as exc:
                 print(f'Failed to connect to {name!r} - '
                       f'{exc.__class__.__name__}: {exc}')
+
         irc.debug('Finished connecting to servers!')
 
+    def wait_until_disconnected(self):
+        for irc in self._prefs:
+            irc.wait_until_disconnected()
+
 # Ensure miniirc isn't outdated
-assert miniirc.ver >= (1,7,0), 'lurklite requires miniirc >= v1.7.0!'
+assert miniirc.ver >= (1,8,0), 'lurklite requires miniirc >= v1.8.0!'
